#!/usr/bin/env python
# -*- coding: utf-8 -*-

# author: Pocholo
# date: 3/20/21

# inspired by a psd tutorial found here: https://textuts.com/scrabble-tiles-text-effect/

# Comments directed to http://gimpchat.com

# Installation:
# This script should be placed in the user plugins folder

# Copyright (C)

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by   
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY# without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <https://www.gnu.org/licenses/>.   

from gimpfu import *
import gettext
import os, sys,time 

gettext.install("gimp20", gimp.locale_directory, unicode=True)

def pm_create_scrabble_tile_text(filename, customtext, fontname, textcolor, size):
          
    #Initiates the temporary state
    pdb.gimp_context_set_defaults()
    pdb.gimp_context_set_default_colors()
	
	#Create the image    
    img = pdb.gimp_image_new(1600, 1600, RGB)
    
    #Open a photo
    imagLayer = pdb.gimp_file_load_layer(img, filename)
    pdb.gimp_image_add_layer(img, imagLayer, 0)   
    pdb.gimp_layer_set_offsets(imagLayer, -1202, -1104)   
    pdb.gimp_layer_resize_to_image_size(imagLayer)
    pdb.gimp_image_set_active_layer(img, imagLayer)
    pdb.gimp_item_set_name(imagLayer, "Background") 
       
    #Image process
    pdb.gimp_display_new(img)
	
    tilelayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Tile Text", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, tilelayer, 0)
    pdb.gimp_layer_add_alpha(tilelayer)   
    pdb.gimp_context_set_pattern("Clean woodV")
    pdb.gimp_drawable_fill(tilelayer, FILL_PATTERN)
    
    #Create the text
    tilelayer = pdb.gimp_image_get_layer_by_name(img, "Tile Text")	
    textW = size/0.3
    textH = size/0.5
    pdb.gimp_context_set_foreground("Black")
    textLayer = pdb.gimp_text_fontname(img, None, 0, 0, customtext, 10, 1, size, PIXELS, fontname)
    pdb.gimp_text_layer_set_justification(textLayer, 2)
    pdb.gimp_text_layer_set_letter_spacing(textLayer, 20)
    pdb.gimp_text_layer_set_line_spacing(textLayer, 0)
        
    #Resize text
    pdb.gimp_layer_scale(textLayer, textW, textH, TRUE)
                
    #Making the text centered	
    textLayer = pdb.gimp_image_get_active_layer(img)
    pdb.gimp_image_set_active_layer(img, textLayer)
    wh = pdb.gimp_drawable_width(textLayer)
    hh = pdb.gimp_drawable_height(textLayer)
    w = pdb.gimp_image_width(img)
    h = pdb.gimp_image_height(img)
    pdb.gimp_image_set_active_layer(img,textLayer)
    centerx=w/2
    centery=h/2
    adw=wh/2
    adh=hh/2
    sw=centerx - adw
    sh=centery - adh		
    pdb.gimp_layer_set_offsets(textLayer, sw, sh)
    pdb.gimp_layer_resize_to_image_size(textLayer)
    
    #Rename the text layer 
    pdb.gimp_item_set_name(textLayer, "Text")
    tilelayer = pdb.gimp_image_merge_down(img, textLayer, CLIP_TO_IMAGE)
    tilelayer = pdb.gimp_image_get_layer_by_name(img, "Tile Text")	
    
    #Create the tiles
    pdb.gimp_fuzzy_select_full(tilelayer, 100, 100, 
                                               100,    #threshold 
                                                 2,    #operation - channel-op-replace
                                              TRUE,    #antialias
                                              TRUE,    #feather
                                                30,    #feather_radius_x
                                                30,    #feather_radius_y
                                             FALSE,    #sample_merged
                                             TRUE,    #select_transparent
                                                 0)    #select_criterion - select-criterion-composite 
    
    #Shapen tiles a little
    pdb.gimp_selection_sharpen(img)
    pdb.gimp_edit_clear(tilelayer)
    pdb.gimp_selection_none(img)
	
	#Text drop shadow
    pdb.python_layerfx_drop_shadow(img, tilelayer, 
                                        (0, 0, 0),   #color
                                               12,   #opacity
                                                0,   #contou - linear
                                                0,   #noise 
                                                3,   #mode for multiply 
                                               60,   #spread 
                                               20,   #size 
                                              120,   #offset_angle 
                                                0,   #offset_distance 
                                             TRUE,   #knockout 
                                            FALSE)   #merge
    
    #Text drop shadow
    pdb.python_layerfx_drop_shadow(img, tilelayer, 
                                        (0, 0, 0),   #color
                                               65,   #opacity
                                                0,   #contou - linear
                                                0,   #noise 
                                                3,   #mode for multiply 
                                                5,   #spread 
                                               30,   #size 
                                              135,   #offset_angle 
                                               30,   #offset_distance 
                                             TRUE,   #knockout 
                                            FALSE)   #merge
    
    #Text copy inner Glow                           
    pdb.python_layerfx_inner_glow(img, tilelayer,
								 (234, 229, 209),   #color
								              75,   #opacity
								               0,   #contou - linear
								               0,   #noise
                                               9,   #mode - soft light
                                               1,   #source
                                              20,   #choke
                                              14,   #size
                                           FALSE)   #merge
		
	#Bevel and emboss
    pdb.python_layerfx_bevel_emboss(img, tilelayer, 
												 1,   #style inner bevel
											    65,   #depth 
												 0,   #direction 
											    12,   #size 
											    12,   #soften
											    90,   #angle 
											    70,   #altitude 
											     0,   #gloss_contour - linear
										   "White",   #highlight_color 
											     5,   #highlight_mode - overlay
											    75,   #highlight_opacity
										   "White",   #shadow_color 
											     5,   #shadow_mode - overlay
											    10,   #shadow_opacity 
												 0,   #surface_contour - linear
											 FALSE,   #use_texture 
									   "Dried mud",   #pattern 
											   100,   #scale 
											   100,   #tex_depth 
											  TRUE,   #invert 
											 FALSE)   #merge	
	        	
	
    backLayer = pdb.gimp_image_get_layer_by_name(img, "Background")
    pdb.gimp_image_set_active_layer(img, backLayer)
        
    #Set gimp to default
    pdb.gimp_context_set_defaults()
    pdb.gimp_context_set_default_colors()
   	
register(
    "pm_create_scrabble_tile_text",
    "Creates a Scrabble tile text",
    "Creates a Scrabble tile text effect",
    "Pocholo",
    "Pocholo",
    "2021",
    "Create a Scrabble Tile Text",
    "",
    [        
    (PF_FILENAME, "filename", "Image:", None),
    (PF_TEXT, "customtext", "Text string", 'TILE\nTEXT\nEFFECT'),
    (PF_FONT, "fontname", "Font name", "Tilez"),
    (PF_COLOR, "textcolor", "Text color:", (0, 0, 0)),
    (PF_SPINNER, "size", "Text size", 420, (300, 420, 1)),
     ],
    [],
    pm_create_scrabble_tile_text, menu="<Image>/Pocholo-scripts/Create a Scrabble Tile Text",
    domain=("gimp20", gimp.locale_directory))
main()						   		